@@ -1,5 +1,13 @@
 #!/usr/bin/perl
-use strict; use warnings;
+#
+# This file is part of POE-Component-SpreadClient
+#
+# This software is copyright (c) 2011 by Apocalypse.
+#
+# This is free software; you can redistribute it and/or modify it under
+# the same terms as the Perl 5 programming language system itself.
+#
+use strict; use warnings FATAL => 'all';
 
 # A sample program that "pings" a group every 2 seconds
 use POE;
